@@ -24,17 +24,21 @@ BEGIN { $fields = {
 	"updated_at" => new WWW::Shopify::Field::Date(min => '2010-01-01 00:00:00', max => 'now'),
 	"tags" => new WWW::Shopify::Field::String::Words(0, 6),
 	"last_order_name" => new WWW::Shopify::Field::String(),
-	"addresses" => new WWW::Shopify::Field::Relation::Many('WWW::Shopify::Model::Address'),
+	"addresses" => new WWW::Shopify::Field::Relation::Many('WWW::Shopify::Model::Address', 0, 3),
 	"metafields" => new WWW::Shopify::Field::Relation::Many("WWW::Shopify::Model::Metafield"),
 	"send_email_invite" => new WWW::Shopify::Field::Boolean(),
 	"password" => new WWW::Shopify::Field::String::Password(),
 	"password_confirmation" => new WWW::Shopify::Field::String::Password()	
 }; }
 
+sub unique_fields { return qw(email); }
+
+sub get_fields { return grep { $_ ne "password" && $_ ne "password_confirmation" && $_ ne "send_email_invite" } keys(%{$_[0]->fields}); }
 sub creation_minimal { return qw(first_name last_name email); }
 sub creation_filled { return qw(created_at); }
 sub update_filled { return qw(updated_at); }
 sub update_fields { return qw(password password_confirmation); }
+sub throws_webhooks { return 1; }
 
 sub searchable($) { return 1; }
 
